#include "biasclocksgui.h"
#include "ui_biasclocksgui.h"

#include "mainwindow.h"
#include "ui_mainwindow.h"

// QThread has static sleep functions; but they're protected (duh).
// So we provide wrapper functions:
//
//    void MyLib::sleep (unsigned long secs)
//    void MyLib::msleep (unsigned long msecs)
//    void MyLib::usleep (unsigned long usecs)
//#include <QThread>
namespace MyLib
{
    class DerivedFromQThread : protected QThread
    {
        public:
            static void sleep (unsigned long secs)   { QThread::sleep (secs) ; }
            static void msleep (unsigned long msecs) { QThread::msleep (msecs) ; }
            static void usleep (unsigned long usecs) { QThread::usleep (usecs) ; }
    } ;
    void sleep (unsigned long secs)   { DerivedFromQThread::sleep (secs) ; }
    void msleep (unsigned long msecs) { DerivedFromQThread::msleep (msecs) ; }
    void usleep (unsigned long usecs) { DerivedFromQThread::usleep (usecs) ; }
} // namespace MyLib

// The bias and voltage params store the raw values that are sent to the ADCs
//
// The order of the data in the array is:
// H+ (0V to +10V=0xfff), H-(-10V=0xfff to 0V), RST+, RST-, OSW+, OSW-, V+, V-, ATG+, ATG- VF+, VF-,
// NEDGE, OTG, VDDR, VDD_OUT, BUF_VDD, BUF_VSS, Spare1, Spare2
// After each of the last 4 parameters are written a serial burst is sent out of the FPGA to the DACs
// in the Clock Module.
// Each DAC input is 16 bits.  The 2 MS bits of data are an address inside the DAC.  The 12 LS bits of data are DAC data.
// When BUF_VDD is written a serial burst is sent out that writes H+, OSW+, TG+, NEDGE, BUF_VDD
// When BUF_VSS is written a serial burst is sent out that writes H-, OSW-, TG-, OTG, BUF_VSS
// When spare1 is written a serial burst is sent out that writes RST+, V+, VF+, VDDR, spare1
// When spare2 is written a serial burst is sent out that writes RST-, V-, VF-, VDD_OUT, spare2
//


//int biasAndVoltageParams [20] = {0x0000, 0x4000, 0x8000, 0xc000, 0x0000, 0x4000, 0x8000, 0xc000, 0x0000, 0x4000, 0x8000, 0xc000, 0x000, 0x4000, 0x8000, 0xc000, 0x000, 0x4000, 0x8000, 0xc000};
//double biasAndVoltageRange [20] = {10.0, -10.0, 10.0, -10.0, 10.0, -10.0, 10.0, -10.0, 10.0, -10.0, 10.0, -10.0, 99.0, 5.0, -15.0, -25.0, -15.0, -15.0, 0.0, 0.0};

//int biasAdrOffset = 0;
//int numBias = 20;

int const pt_posH       = 0;
int const pt_negH       = 1;
int const pt_posRG      = 2;
int const pt_negRG      = 3;
int const pt_posSW      = 4;
int const pt_negSW      = 5;
int const pt_posV       = 6;
int const pt_negV       = 7;
int const pt_posTG      = 8;
int const pt_negTG      = 9;
int const pt_posVF      = 10;
int const pt_negVF      = 11;
int const pt_NEDGE      = 12;
int const pt_OTG        = 13;
int const pt_VDDR       = 14;
int const pt_VDD_OUT    = 15;
int const pt_BUF_Base   = 16;
int const pt_BUF_Delta  = 17;
int const pt_Spare1     = 18;
int const pt_Spare2     = 19;

//int RTDChip1Offset = 4096;
//int RTDChip1ModeReg = 0;

////int RTDChip2Offset  = 4100;
int const RTDMSReg[2]   = {0x28, 0x2c};
int const RTDLSReg[2]   = {0x2a, 0x2e};
int const RTDReadReg[2] = {0x58, 0x5c};


//int cin->clock_control_reg = 8;
//
//

biasclocksgui::biasclocksgui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::biasclocksgui)
{
    ui->setupUi(this);
    ui->labelBiasOn->setVisible(0);
    biasOn = 0;
    ui->labelClocksOn->setVisible(0);
    clocksOn = 0;
    ui->labelBiasesToCin->setVisible(0);

    biasAndVoltageParams[0]   = 0x0000;
    biasAndVoltageParams[1]   = 0x4000;
    biasAndVoltageParams[2]   = 0x8000;
    biasAndVoltageParams[3]   = 0xc000;
    biasAndVoltageParams[4]   = 0x0000;
    biasAndVoltageParams[5]   = 0x4000;
    biasAndVoltageParams[6]   = 0x8000;
    biasAndVoltageParams[7]   = 0xc000;
    biasAndVoltageParams[8]   = 0x0000;
    biasAndVoltageParams[9]   = 0x4000;
    biasAndVoltageParams[10]  = 0x8000;
    biasAndVoltageParams[11]  = 0xc000;
    biasAndVoltageParams[12]  = 0x0000;
    biasAndVoltageParams[13]  = 0x4000;
    biasAndVoltageParams[14]  = 0x8000;
    biasAndVoltageParams[15]  = 0xc000;
    biasAndVoltageParams[16]  = 0x0000;
    biasAndVoltageParams[17]  = 0x4000;
    biasAndVoltageParams[18]  = 0x8000;
    biasAndVoltageParams[19]  = 0xc000;

    biasAndVoltageRange[0]    = 9.0;  //10.0;
    biasAndVoltageRange[1]    = -9.0; //-10.0;
    biasAndVoltageRange[2]    = 9.0;  //10.0;
    biasAndVoltageRange[3]    = -9.0; //-10.0;
    biasAndVoltageRange[4]    = 9.0;  //10.0;
    biasAndVoltageRange[5]    = -9.0; //-10.0;
    biasAndVoltageRange[6]    = 9.0;  //10.0;
    biasAndVoltageRange[7]    = -9.0; //-10.0;
    biasAndVoltageRange[8]    = 9.0;  //10.0;
    biasAndVoltageRange[9]    = -9.0; //-10.0;
    biasAndVoltageRange[10]   = 9.0;  //10.0;
    biasAndVoltageRange[11]   = -9.0; //-10.0;
    biasAndVoltageRange[12]   = 99.0;
    biasAndVoltageRange[13]   = 5.0;
    biasAndVoltageRange[14]   = -15.0;
    biasAndVoltageRange[15]   = -25.0;
    biasAndVoltageRange[16]   = -10.0;
    biasAndVoltageRange[17]   = -5.1;
    biasAndVoltageRange[18]   = 0.0;
    biasAndVoltageRange[19]   = 0.0;

    biasAdrOffset = 0;
    numBias = 20;

    RTDChip1Offset = 4096;
    RTDChip1ModeReg = 0;

    //----readRTD is not working, so I am hiding it from the gui---
    ui->label_56->setVisible(0);
    ui->pushButtonReadRTD1->hide();
    ui->pushButtonReadRTD2->hide();
    ui->pushButtonReadRTD3->hide();
    ui->pushButtonReadRTD4->hide();
    ui->RTD1ValuelineEdit->hide();
    ui->RTD2ValuelineEdit->hide();
    ui->RTD3ValuelineEdit->hide();
    ui->RTD4ValuelineEdit->hide();

}

biasclocksgui::~biasclocksgui()
{
    delete ui;
}


//============================================================================
//              General Functions
//============================================================================
// This routine takes in reg info and returns the RTD's ADC value
//
// All comunications to RTD chip start with a write to the 8 bit communicatons register
// This is followed by 0, 8 or 16 bits of data.
// So each communication is either 8,16, or 24 bits if you combind the communication register data with the following data
// The FPGA has the following address that are written into to perform writes:
//
// RTD Chip1 Addresses
// 0x28     MS Byte = gives number of bits to stream to RTD (8,16, or 24) in hex (0x08, 0x10, 0x18)
//          LS Byte = MS bits to be sent to RTD Bits[23 downto 16] (Only used with 24 bit transfer)
// 0x2a     Rest of bits to be sent to Chip1 Bits[15 downto 0] (15 downto 8 only used with 24 and 16 bit options)
// Data only sent to RTD chip after write to 2a
//
// Address 2e, and 2c are the same for the RTC chip2
//

// ChipNum = 0 or 1, ADCNum = 0 or 1
//int biasclocksgui::readRTDValue(int ChipNum, int ADCNum)
//{
    /*

    QByteArray datagram;
    console->append("Reading RDT");

    // send a string of ones to reset AD7792
    datagram.clear();
    append16(datagram, RTDMSReg[ChipNum], 0x3000);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    datagram.clear();
    append16(datagram, RTDLSReg[ChipNum], 0x0000);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;


    // Initialize I/O register
    // Write 8 bit communication register followed by 8 bit I/O register
    datagram.clear();
    append16(datagram, RTDMSReg[ChipNum], 0x1000);  // 16 bits to be streamed to RTD chip
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    datagram.clear();
    append16(datagram, RTDLSReg[ChipNum], 0x2803);  // communication reg data followed by I/O reg data
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;

    // Initialize Configuration Register
    // Write 8 bit communication register followed by 16 bit configuration register
    // VBias = Disabled, BO = Off, Boost=Off, Polarity = Uni, Ref = Internal, Buffer = 0ff, Gain = 2x, Channel(ADC1 = 0, ADC2 = 1)
    // ADC1 = 1180 or ADC2 = 1181
    append16(datagram, RTDMSReg[ChipNum], 0x1810);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    datagram.clear();
    append16(datagram, RTDLSReg[ChipNum], 0x1180 + ADCNum);     // orginal was 16x with buffer on = 1490
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;


    // Write Mode Register
    // Write 8 bit communication register followed by 16 bit mode register
    // Single conversion
    append16(datagram, RTDMSReg[ChipNum], 0x1808);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    datagram.clear();
    append16(datagram, RTDLSReg[ChipNum], 0x200A);     // orginal was 16x with buffer on = 1490
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;


    // Write 8 bit communication register to start read
    append16(datagram, RTDMSReg[ChipNum], 0x0800);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    datagram.clear();
    append16(datagram, RTDLSReg[ChipNum], 0x0058);     // orginal was 16x with buffer on = 1490
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;

    // After a delay Send out command to read register with last ADC conversion
    // Before this command is sent out I should clear any existing datagrams
    //
    datagram.clear();
    append16(datagram, "0002" , RTDReadReg[ChipNum]);
    // ???????????? is this needed?
    appendHex(datagram,"0001","0001");

    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                             *cameraControlIP, *cameraControlPortNum) << endl;
    qDebug() << "IP: " << *cameraControlIP.toString() << " " << *cameraControlPortNum;

    //Read in returned data gram
    //May need a delay or a loop waiting for responce
    //
    QByteArray datagramIn;
    //QByteArray line;
    datagramIn.resize(udpSocket->pendingDatagramSize());
    QHostAddress sender;
    quint16 senderPort;
    int dataIn=0;

    if ( udpSocket->pendingDatagramSize()  == -1)
    {
        qDebug() << "No datagramIn available";
        dataIn = -1;
    }
    else
    {
        qDebug() << "Reading datagramIn";
        udpSocket->readDatagram(datagramIn.data(), datagramIn.size(), &sender, &senderPort);
        // assuming only one address and data is returned this should be an 8 byte message
        // the first 4 bytes are for address and the last 4 bytes are the data
        //
        dataIn = (datagram.at(4) * 0x1000) + (datagram.at(5) * 0x100) + (datagram.at(6) * 0x10) + datagram.at(7);
    }
    return (dataIn);
    */

  //  return 0; // bypass added by pat
//}



void biasclocksgui::setBiasOn()
{
    ui->pushButtonBiasOn->click();
}
void biasclocksgui::setBiasOff()
{
    ui->pushButtonBiasOFF->click();
}
void biasclocksgui::setClocksOn()
{
    ui->pushButtonClockON->click();
}
void biasclocksgui::setClocksOff()
{
    ui->pushButtonClockOFF->click();
}

void biasclocksgui::UpdateGUIBiasOnOff( int arg1 )
{
   // if (arg1 == 1) ui->pushButtonBiasOn->click();
   // else ui->pushButtonBiasOFF->click();

    if ( !(arg1 == 1 || arg1 == 0) )
    {
        // error
        console->append("ERROR: Invalid Bias Off/On Command");
    }

   if ( (arg1 & 0x1) == 0x1)
    {
        console->append("Bias On");
        biasOn = 1;
        ui->labelBiasOff->setVisible(0);
        ui->labelBiasOn->setVisible(1);
    }
   else
   {
       console->append("Bias Off");
       biasOn = 0;
       ui->labelBiasOff->setVisible(1);
       ui->labelBiasOn->setVisible(0);
   }


}
void biasclocksgui::UpdateGUIClockOnOff( int arg1 )
{
    if ( (arg1 & 0x1) == 0x0)
    {
        console->append("Clocks Off");
        clocksOn = 0;
        ui->labelClocksOff->setVisible(1);
        ui->labelClocksOn->setVisible(0);

        cin->clock_control_reg = cin->clock_control_reg & 0xfffe;
        //ui->pushButtonClockOFF->click();
    }
    else if ( (arg1 & 0x1) == 0x1)
    {
        console->append("Clocks On");
        clocksOn = 1;
        ui->labelClocksOff->setVisible(0);
        ui->labelClocksOn->setVisible(1);

        cin->clock_control_reg = cin->clock_control_reg | 0x0001;
        //ui->pushButtonClockON->click();
    }

    // we used to check for exposure mode or focus bit here. This doesn't make sense because exposure mode
    // has little to nothing to do with the clock being on or off other than everything being stuffed into
    // the same register. I moved it to the trigger gui.
    //The new function is called UpdateGUITriggerExposureSettings( int arg1 )

    console->append(QLatin1String("Bias:" ) + QString("%1").arg(cin->clock_control_reg));
}
int biasclocksgui::GetPushButtonBiasOnOff()
{
    return biasOn;
}

int biasclocksgui::GetPushButtonClockOnOff()
{
    return clocksOn;
}

void biasclocksgui::updateBiasGUIWithBiasParams()
{
    int ii;
    float eng_value;

    for (ii=0; ii < numBias; ii++)
    {
        eng_value = float( (biasAndVoltageParams[ii] & 0xfff) * biasAndVoltageRange[ii]) / 4096.0;
//        qDebug()<< "preval1:  "<< float(biasAndVoltageParams[ii] & 0xfff);
//        qDebug()<< "preval1:  "<< float( (biasAndVoltageParams[ii] & 0xfff) * biasAndVoltageRange[ii]);
        switch (ii)
        {
        case pt_posH:
            ui->doubleSpinBoxPosH->setValue(eng_value);
//            qDebug()<< "the eng value for addr 0 i.e. posH is: "<< eng_value;
            break;

        case pt_negH:
            ui->doubleSpinBoxNegH->setValue(eng_value);
//            qDebug()<< "the eng value for addr 1 i.e. negH is: "<< eng_value;
            break;

        case pt_posRG:
            ui->doubleSpinBoxPosRG->setValue(eng_value);
//            qDebug()<< "the eng value for addr 2 i.e. posRG is: "<< eng_value;
            break;

        case pt_negRG:
            ui->doubleSpinBoxNegRG->setValue(eng_value);
//            qDebug()<< "the eng value for addr 3 i.e. negRG is: "<< eng_value;
            break;

        case pt_posSW:
            ui->doubleSpinBoxPosSW->setValue(eng_value);
//            qDebug()<< "the eng value for addr 4 i.e. posSW is: "<< eng_value;
            break;

        case pt_negSW:
            ui->doubleSpinBoxNegSW->setValue(eng_value);
//            qDebug()<< "the eng value for addr 5 i.e. negSW is: "<< eng_value;
            break;

        case pt_posV:
            ui->doubleSpinBoxPosV->setValue(eng_value);
//            qDebug()<< "the eng value for addr 6 i.e. posV is: "<< eng_value;
            break;

        case pt_negV:
            ui->doubleSpinBoxNegV->setValue(eng_value);
//            qDebug()<< "the eng value for addr 7 i.e. negV is: "<< eng_value;
            break;

        case pt_posTG:
            ui->doubleSpinBoxPosTG->setValue(eng_value);
//            qDebug()<< "the eng value for addr 8 i.e. posTG is: "<< eng_value;
            break;

        case pt_negTG:
            ui->doubleSpinBoxNegTG->setValue(eng_value);
//            qDebug()<< "the eng value for addr 9 i.e. negTG is: "<< eng_value;
            break;

        case pt_posVF:
            ui->doubleSpinBoxPosVF->setValue(eng_value);
//            qDebug()<< "the eng value for addr 10 i.e. posVF is: "<< eng_value;
            break;

        case pt_negVF:
            ui->doubleSpinBoxNegVF->setValue(eng_value);
//            qDebug()<< "the eng value for addr 11 i.e. negVF is: "<< eng_value;
            break;

        case pt_NEDGE:
            ui->doubleSpinBoxBias->setValue(eng_value);
//            qDebug()<< "the eng value for addr 12 i.e. NEDGE is: "<< eng_value;
            break;

        case pt_OTG:
            ui->doubleSpinBoxOTG->setValue(eng_value);
//            qDebug()<< "the eng value for addr 13 i.e. posOTG is: "<< eng_value;
            break;

        case pt_VDDR:
            ui->doubleSpinBoxVDDReset->setValue(eng_value);
//            qDebug()<< "the eng value for addr 14 i.e. VDDR is: "<< eng_value;
            break;

        case pt_VDD_OUT:
            ui->doubleSpinBoxVDDOut->setValue(eng_value);
//            qDebug()<< "the eng value for addr 15 i.e. VDD_OUT is: "<< eng_value;
            break;

        case pt_BUF_Base:
            ui->doubleSpinBoxBUFVDD->setValue(eng_value);
//            qDebug()<< "the eng value for addr 16 i.e. BUF_Base is: "<< eng_value;
            break;

        case pt_BUF_Delta:
            ui->doubleSpinBoxBUFVSS->setValue(eng_value);
//            qDebug()<< "the eng value for addr 17 i.e. Buf_Delta is: "<< eng_value;
            break;
        }
        updateBiasGUIWithReadBackParams(); 
    }
}

void biasclocksgui::updateBiasGUIWithReadBackParams()
{
    // When the serial burst is sent out, it is done twice.
    // this allows the read back registers in the FPGA to be update with what was sent down
    // These commands read the FPGA Bias read back registers and writes them to the GUI
    // Readback registers start at address 0x30

    int ii;
    int readAddress;
    float eng_value;

    for (ii=0; ii < numBias; ii++)
    {
        QByteArray datagram;
        QByteArray lastDatagramReadHex;
        long lastDatagramReadInt;
        //QByteArray line;
        //QHostAddress sender;
        //quint16 senderPort;
        bool ok;

        qDebug() << endl;

        datagram.clear();
        readAddress = 0x30 + (2*ii);
        append16(datagram, 0x8200, readAddress );      // address to read
        qDebug() << "writeDatagram, address to read back:  0x8200" << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                     *cameraControlIP, *cameraControlPortNum) << readAddress;
        datagram.clear();
        appendHex(datagram, "0002", "8203");               // read bias and clock read register
        appendHex(datagram, "0001", "0001");
        qDebug() << "writeDatagram, read back command: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                     *cameraControlIP, *cameraControlPortNum);

        // Wait here and then let readPendingDataCommandDatagrams() put data into lastDatagramRead
        if (commandSocket->waitForReadyRead(3000))
        {
//            datagram.resize(commandSocket->pendingDatagramSize());
//            commandSocket->readDatagram(lastDatagramRead.data(), lastDatagramRead.size(),
//                                     &sender, &senderPort);
//            qDebug() << "Data received";
        }
        else
        {
            qDebug() << "ERROR: No Data Read Back";
        }

        // convert readback Qbytearray into hex array
        lastDatagramReadHex = lastDatagramRead;
        lastDatagramReadHex = lastDatagramReadHex.toHex();
        lastDatagramReadInt = lastDatagramReadHex.toLong(&ok,16);
        lastDatagramReadInt = lastDatagramReadInt & 0x3fff;

        //qDebug() << "lastDatagramReadHex Size, Data: " << lastDatagramReadHex.size() << lastDatagramReadHex.data();
        qDebug() << "lastDatagramReadInt: " << lastDatagramReadInt;

        eng_value = float ( lastDatagramReadInt * biasAndVoltageRange[ii]) / 4096.0;
        qDebug() << "eng_value: " << eng_value;

        switch (ii)
        {
        case pt_posH:
            ui->lineEditPosH->clear();
            ui->lineEditPosH->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negH:
            ui->lineEditNegH->clear();
            ui->lineEditNegH->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_posRG:
            ui->lineEditPosRG->clear();
            ui->lineEditPosRG->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negRG:
            ui->lineEditNegRG->clear();
            ui->lineEditNegRG->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_posSW:
            ui->lineEditPosSW->clear();
            ui->lineEditPosSW->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negSW:
            ui->lineEditNegSW->clear();
            ui->lineEditNegSW->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_posV:
            ui->lineEditPosV->clear();
            ui->lineEditPosV->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negV:
            ui->lineEditNegV->clear();
            ui->lineEditNegV->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_posTG:
            ui->lineEditPosTG->clear();
            ui->lineEditPosTG->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negTG:
            ui->lineEditNegTG->clear();
            ui->lineEditNegTG->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_posVF:
            ui->lineEditPosVF->clear();
            ui->lineEditPosVF->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_negVF:
            ui->lineEditNegVF->clear();
            ui->lineEditNegVF->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_NEDGE:
            ui->lineEditBias->clear();
            ui->lineEditBias->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_OTG:
            ui->lineEditOTG->clear();
            ui->lineEditOTG->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_VDDR:
            ui->lineEditVDDReset->clear();
            ui->lineEditVDDReset->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_VDD_OUT:
            ui->lineEditVDDOut->clear();
            ui->lineEditVDDOut->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_BUF_Base:
            ui->lineEditBUFVdd->clear();
            ui->lineEditBUFVdd->insert( QString::number( eng_value, 10, 2 ) );
            break;

        case pt_BUF_Delta:
            ui->lineEditBUFDelta->clear();
            ui->lineEditBUFDelta->insert( QString::number( eng_value, 10, 2 ) );
            break;
        }
    }
}

//============================================================================
//              Push Buttons
//============================================================================


// To Write data into BiasAndVoltage array
void biasclocksgui::setBiasAndVoltageParams( int address, int data)
{
    biasAndVoltageParams [ address ] = data;
}

void biasclocksgui::setLastDatagramRead (QByteArray datagram)
{
    lastDatagramRead = datagram;
}

void biasclocksgui::SetSendBiasToCINpushButton_2_click()
{
    biasclocksgui::on_SendBiasToCINpushButton_2_clicked();
}

void biasclocksgui::on_SaveBiasToFilepushButton_clicked()
{
    QFile iofile;
    QString fileName;
    QByteArray line;
    QString str;

    int ii;

    console->append("Saved to file.");
    fileName = QFileDialog::getSaveFileName(this,
         tr("Save Bias settings to file"), "/home/bias_setting.txt", tr("Txt Files (*.txt);;Any(*.*)"));

    iofile.setFileName(fileName);
    if(!iofile.open(QIODevice::WriteOnly)){
        console->append("Could not open file.");
        return;
    }
    line = "# Bias settings File (Do not edit this line)\r\n";
    if(!iofile.write(line)){
        console->append("File write error.");
        iofile.close();
        return;
    }
    line = "#\r\n";
    iofile.write(line);
    line = "#\r\n";
    iofile.write(line);
    line = "# There are 20 Bias Voltages to be set\r\n";
    iofile.write(line);
    line = "# There are 5 DACs with 4 outputs each.\r\n";
    iofile.write(line);
    line = "# The DACs are daisy chanined so which DAC you are writting is determined by the word position.\r\n";
    iofile.write(line);
    line = "# Each DAC is a 16 bit number.\r\n";
    iofile.write(line);
    line = "# The 2 MS bits determine which of the 4 outpus in a specific DAC is written\r\n";
    iofile.write(line);
    line = "# and the LS 12 bits are the DAC value\r\n";
    iofile.write(line);
    line = "# JTW 10/10/2012 - Modified file format by Adding 0x8000 to all address\r\n";
    iofile.write(line);
    line = "#\r\n";
    iofile.write(line);

    for (ii=0; ii < numBias; ii++)
    {
        line = "";
        str = "%1";
        //str = "%L4";
        // Address
        line.append (str.arg("8200"));
        line.append (" ");
        line.append (str.arg( ((ii * 2) + biasAdrOffset), 4, 16, QLatin1Char('0')));
        // line.append (str.arg( (ii * 2) + biasAdrOffset) );
        line.append ("\r\n");
        //iofile.write(line);

        // Data
        line.append (str.arg("8201"));
        line.append (" ");
        line.append (str.arg( biasAndVoltageParams [ii], 4, 16, QLatin1Char('0')));
        line.append ("\r\n");
        //iofile.write(line);

        // Command to write Bias
        line.append (str.arg("8001"));
        line.append (" ");
        line.append (str.arg("0102") );
        line.append ("\r\n");
        iofile.write(line);
    }
    iofile.close();
}



void biasclocksgui::on_pushButtonBiasOFF_clicked()
{

    QByteArray datagram;


    // Update GUI
    ui->labelBiasOff->setVisible(1);
    ui->labelBiasOn->setVisible(0);
    biasOn = 0;

    // send command out
    datagram.clear();
    appendHex(datagram, "8204", "0000" );
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                 *cameraControlIP, *cameraControlPortNum) << endl;
    MyLib::msleep(1);
    //qDebug() << "IP: " << *cameraControlIP.toString() << " " << *cameraControlPortNum;
}

void biasclocksgui::on_pushButtonBiasOn_clicked()
{

    QByteArray datagram;


    // Update GUI
    ui->labelBiasOff->setVisible(0);
    ui->labelBiasOn->setVisible(1);
    biasOn = 1;

    // send command out
    datagram.clear();
    appendHex(datagram, "8204", "0001" );
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                 *cameraControlIP, *cameraControlPortNum) << endl;
    MyLib::msleep(1);
    //qDebug() << "IP: " << *cameraControlIP.toString() << " " << *cameraControlPortNum;
}
void biasclocksgui::updateBiasONGUIRemote()
{
    biasclocksgui::on_pushButtonBiasOn_clicked();
}
void biasclocksgui::on_pushButtonClockOFF_clicked()
{

    QByteArray datagram;

    // Update GUI
    ui->labelClocksOff->setVisible(1);
    ui->labelClocksOn->setVisible(0);
    clocksOn = 0;

    // send command out
    // clear clocks enable bit first
    cin->clock_control_reg = cin->clock_control_reg & 0xfffe;

    datagram.clear();
    //appendHex(datagram, "8205", "0000" );
    append16(datagram, 0x8205, cin->clock_control_reg);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                 *cameraControlIP, *cameraControlPortNum) << endl;
    MyLib::msleep(1);
    //qDebug() << "IP: " << *cameraControlIP.toString() << " " << *cameraControlPortNum;
}
void biasclocksgui::updateBiasOFFGUIRemote()
{
    biasclocksgui::on_pushButtonBiasOFF_clicked();
}
void biasclocksgui::on_pushButtonClockON_clicked()
{

    QByteArray datagram;


    // Update GUI
    ui->labelClocksOff->setVisible(0);
    ui->labelClocksOn->setVisible(1);
    clocksOn = 1;

    // send command out
    // set clock enable bit first
    cin->clock_control_reg = cin->clock_control_reg | 0x0001;
    // Second: build command and send out
    datagram.clear();
    // appendHex(datagram, "8205", "0001" );
    append16(datagram, 0x8205, cin->clock_control_reg);
    qDebug() << "writeDatagram: " << commandSocket->writeDatagram(datagram.data(), datagram.size(),
                                 *cameraControlIP, *cameraControlPortNum) << endl;
    MyLib::msleep(1);
    //qDebug() << "IP: " << *cameraControlIP.toString() << " " << *cameraControlPortNum;
}
void biasclocksgui::updateClockONGUIRemote()
{
    biasclocksgui::on_pushButtonClockON_clicked();
}
void biasclocksgui::updateClockOFFGUIRemote()
{
    biasclocksgui::on_pushButtonClockOFF_clicked();
}
//----readRTDValue is either non-functional or deprecated because the actual function-----
//----has been bypassed, so I've hidden all the gui elements and turn off the functions.--

//void biasclocksgui::on_pushButtonReadRTD1_clicked()
//{
//    ui->RTD1ValuelineEdit->setText( QString::number(readRTDValue(0, 0)));
//}

//void biasclocksgui::on_pushButtonReadRTD2_clicked()
//{
//    ui->RTD2ValuelineEdit->setText( QString::number(readRTDValue(0, 1)) );
//}

//void biasclocksgui::on_pushButtonReadRTD3_clicked()
//{
  //  ui->RTD3ValuelineEdit->setText( QString::number(readRTDValue(1, 0)) );
//}

//void biasclocksgui::on_pushButtonReadRTD4_clicked()
//{
//    ui->RTD4ValuelineEdit->setText( QString::number(readRTDValue(1, 0)) );
//}

void biasclocksgui::on_SendBiasToCINpushButton_2_clicked()
{
    // send out parameter list in Bias data array, will not put into a file
    //
    // When registers 16, 17, 18, and/or 19 (0x20, 0x22, 0x24, 0x26) are written
    // this initiates a burst of serial data to the DACs.
    // During this time the GUI needs to pause while the data is sent down.
    // The rest of the registers just update a register inside the FPGA.

    QByteArray datagram;

    int ii;

    // 3-01-2013 JTW no changes if clocks or bias are on
    //if ( (ui->labelBiasOn->isVisible() == 1)  || (ui->labelClocksOn->isVisible() == 1  ) )
    if ( (biasOn == 1) || (clocksOn == 1) )
    {
        QMessageBox::information(0, QString("Warning"), QString("You cannot update voltages with clocks or biases on!"), QMessageBox::Ok);
        return;
    }

    for (ii=0; ii <= numBias; ii++)
    {
        // Address
        datagram.clear();
        append16(datagram, 32768 + 512, ii*2);
        commandSocket->writeDatagram(datagram.data(), datagram.size(), *cameraControlIP, *cameraControlPortNum);
        MyLib::msleep(1);

        // Data
        datagram.clear();
        append16(datagram, 32768 + 513, biasAndVoltageParams[ii] );
        commandSocket->writeDatagram(datagram.data(), datagram.size(), *cameraControlIP, *cameraControlPortNum);
        MyLib::msleep(1);

        // Command to write Bias Data
        datagram.clear();
        //append16(datagram, 1, 259);
        appendHex(datagram, "8001", "0102");
        commandSocket->writeDatagram(datagram.data(), datagram.size(), *cameraControlIP, *cameraControlPortNum);
        //MyLib::msleep(1);

        // 3-23-2012 jtw change from 200 to 100 to speed up.  Hopefully this will be enough time between ADC burst -- PJM changed
        // 1-15-2012 Put back by JTW!
        //MyLib::msleep(100);

        // 3-15-2013 jtw only do delay for registers that send down serial burst and delay for 30msec
        if (ii == 16 || ii == 17 || ii == 18 || ii == 19)
        {
            MyLib::msleep(30);
            // read read back registers and update screen
            updateBiasGUIWithReadBackParams();
        }
        else MyLib::msleep(2);
    }
    ui->labelBiasesToCin->setVisible(0);
}
void biasclocksgui::sendBiasToCINGUIRemote()
{
    biasclocksgui::on_SendBiasToCINpushButton_2_clicked();
}
//============================================================================
//              Spin Box Value Changes
//============================================================================

void biasclocksgui::on_doubleSpinBoxBias_valueChanged(double arg1)
{
    // Input range is 0 to 99V.  This needs to be converted to 0 to 0xfff and stored in the last 12 bits in the DAC Data array.
    biasAndVoltageParams [pt_NEDGE] = (biasAndVoltageParams [pt_NEDGE] & 0xc000) + ((arg1/biasAndVoltageRange[pt_NEDGE]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_NEDGE << "Data_pre: " << biasAndVoltageParams [pt_NEDGE] << "Data: " << (biasAndVoltageParams [pt_NEDGE] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateBiasGUIRemote(int addr, int data)
{
    qDebug() << "Remote Bias Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_NEDGE] & 0xfff) * biasAndVoltageRange[pt_NEDGE]) / 4096.0;
    qDebug() << "engval is: " << eng_value;
    ui->doubleSpinBoxBias->setValue(eng_value);

}
void biasclocksgui::updateBiasGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_NEDGE] = (biasAndVoltageParams [pt_NEDGE] & 0xc000) + ((arg1/biasAndVoltageRange[pt_NEDGE]) * 0xfff);
    qDebug() << "Calculate Bias and Voltage Array    Address: " << pt_NEDGE << "DatawithADR: " << biasAndVoltageParams [pt_NEDGE] << "Data: " << (biasAndVoltageParams [pt_NEDGE] & 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_NEDGE] & 0xfff) * biasAndVoltageRange[pt_NEDGE]) / 4096.0;
    ui->doubleSpinBoxBias->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxOTG_valueChanged(double arg1)
{
    // convert to raw ADC value from engineering unints and store raw
    biasAndVoltageParams [pt_OTG] = (biasAndVoltageParams [pt_OTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_OTG]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array Address: " << pt_OTG << "DatawithADR: " << biasAndVoltageParams [pt_OTG] << "Data: " << (biasAndVoltageParams [pt_OTG] & 0xfff);;
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateOTGGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_OTG] & 0xfff) * biasAndVoltageRange[pt_OTG]) / 4096.0;
    ui->doubleSpinBoxOTG->setValue(eng_value);

}
void biasclocksgui::updateOTGGUIRemote2(double arg1)
{
        biasAndVoltageParams [pt_OTG] = (biasAndVoltageParams [pt_OTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_OTG]) * 0xfff);
        float eng_value = float( (biasAndVoltageParams[pt_OTG] & 0xfff) * biasAndVoltageRange[pt_OTG]) / 4096.0;
        ui->doubleSpinBoxOTG->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxVDDReset_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_VDDR] = (biasAndVoltageParams [pt_VDDR] & 0xc000) + ((arg1/biasAndVoltageRange[pt_VDDR]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_VDDR << "DatawithADR: " << biasAndVoltageParams [pt_VDDR] << "Data: " << (biasAndVoltageParams [pt_VDDR] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateVDDResetGUIRemote(int addr, int data)
{
    qDebug() << "Remote VDDReset Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_VDDR] & 0xfff) * biasAndVoltageRange[pt_VDDR]) / 4096.0;
    ui->doubleSpinBoxVDDReset->setValue(eng_value);

}
void biasclocksgui::updateVDDResetGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_VDDR] = (biasAndVoltageParams [pt_VDDR] & 0xc000) + ((arg1/biasAndVoltageRange[pt_VDDR]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_VDDR] & 0xfff) * biasAndVoltageRange[pt_VDDR]) / 4096.0;
    ui->doubleSpinBoxVDDReset->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxVDDOut_valueChanged(double arg1)
{
    // convert to raw unints and store
    biasAndVoltageParams [pt_VDD_OUT] = (biasAndVoltageParams [pt_VDD_OUT] & 0xc000) + ((arg1/biasAndVoltageRange[pt_VDD_OUT]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_VDD_OUT << "DatawithADR: " << biasAndVoltageParams [pt_VDD_OUT] << "Data: " << (biasAndVoltageParams [pt_VDD_OUT] & 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_VDD_OUT << "DatawithADR: " << biasAndVoltageParams [pt_VDD_OUT] << "Data: " << (biasAndVoltageParams [pt_VDD_OUT] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateVDDOutGUIRemote(int addr, int data)
{
    qDebug() << "Remote VDDOut Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_VDD_OUT] & 0xfff) * biasAndVoltageRange[pt_VDD_OUT]) / 4096.0;
    ui->doubleSpinBoxVDDOut->setValue(eng_value);

}
void biasclocksgui::updateVDDOutGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_VDD_OUT] = (biasAndVoltageParams [pt_VDD_OUT] & 0xc000) + ((arg1/biasAndVoltageRange[pt_VDD_OUT]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_VDD_OUT] & 0xfff) * biasAndVoltageRange[pt_VDD_OUT]) / 4096.0;
    ui->doubleSpinBoxVDDOut->setValue(eng_value);
}

// VDD is now BASE
void biasclocksgui::on_doubleSpinBoxBUFVDD_valueChanged(double arg1)
{
    biasAndVoltageParams [pt_BUF_Base] = (biasAndVoltageParams [pt_BUF_Base] & 0xc000) + ((arg1/biasAndVoltageRange[pt_BUF_Base]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_BUF_Base << "DatawithADR: " << biasAndVoltageParams [pt_BUF_Base] << "Data: " << (biasAndVoltageParams [pt_BUF_Base] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);

    // Check that change is allowed.
    // 1 - Must be greater than Vdd_OUT
//    double Buf_Vss = ( (biasAndVoltageParams[pt_BUF_Delta] & 0xfff) * biasAndVoltageRange[pt_BUF_Delta])/0xfff;
//    double Vdd_Out = ( (biasAndVoltageParams[pt_VDD_OUT] & 0xfff) * biasAndVoltageRange[pt_VDD_OUT])/0xfff;

//    if ( arg1 > Vdd_Out )
//    {
//    }

}

void biasclocksgui::updateBUFVDDGUIRemote(int addr, int data)
{
    qDebug() << "Remote VDDOut Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_BUF_Base] & 0xfff) * biasAndVoltageRange[pt_BUF_Base]) / 4096.0;
    ui->doubleSpinBoxBUFVDD->setValue(eng_value);

}
void biasclocksgui::updateBUFVDDGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_BUF_Base] = (biasAndVoltageParams [pt_BUF_Base] & 0xc000) + ((arg1/biasAndVoltageRange[pt_BUF_Base]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_BUF_Base] & 0xfff) * biasAndVoltageRange[pt_BUF_Base]) / 4096.0;
    ui->doubleSpinBoxBUFVDD->setValue(eng_value);
}

// VSS is now BUF_Delta
void biasclocksgui::on_doubleSpinBoxBUFVSS_valueChanged(double arg1)
{
    biasAndVoltageParams [pt_BUF_Delta] = (biasAndVoltageParams [pt_BUF_Delta] & 0xc000) + (( arg1/biasAndVoltageRange[pt_BUF_Delta]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_BUF_Delta << "DatawithADR: " << biasAndVoltageParams [pt_BUF_Delta] << "Data: " << (biasAndVoltageParams [pt_BUF_Delta] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}
void biasclocksgui::updateBUFVSSGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_BUF_Delta] & 0xfff) * biasAndVoltageRange[pt_BUF_Delta]) / 4096.0;
    ui->doubleSpinBoxBUFVSS->setValue(eng_value);
}
void biasclocksgui::updateBUFVSSGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_BUF_Delta] = (biasAndVoltageParams [pt_BUF_Delta] & 0xc000) + (( arg1/biasAndVoltageRange[pt_BUF_Delta]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_BUF_Delta] & 0xfff) * biasAndVoltageRange[pt_BUF_Delta]) / 4096.0;
    ui->doubleSpinBoxBUFVSS->setValue(eng_value);
}


void biasclocksgui::on_doubleSpinBoxNegV_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negV] = (biasAndVoltageParams [pt_negV] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negV]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negV << "DatawithADR: " << biasAndVoltageParams [pt_negV] << "Data: " << (biasAndVoltageParams [pt_negV] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateNegVGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negV] & 0xfff) * biasAndVoltageRange[pt_negV]) / 4096.0;
    ui->doubleSpinBoxNegV->setValue(eng_value);

}

void biasclocksgui::updateNegVGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negV] = (biasAndVoltageParams [pt_negV] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negV]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negV] & 0xfff) * biasAndVoltageRange[pt_negV]) / 4096.0;
    ui->doubleSpinBoxNegV->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosV_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posV] = (biasAndVoltageParams [pt_posV] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posV]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posV << "DatawithADR: " << biasAndVoltageParams [pt_posV] << "Data: " << (biasAndVoltageParams [pt_posV] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updatePosVGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posV] & 0xfff) * biasAndVoltageRange[pt_posV]) / 4096.0;
    ui->doubleSpinBoxPosV->setValue(eng_value);
}
void biasclocksgui::updatePosVGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posV] = (biasAndVoltageParams [pt_posV] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posV]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posV] & 0xfff) * biasAndVoltageRange[pt_posV]) / 4096.0;
    ui->doubleSpinBoxPosV->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxNegVF_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negVF] = (biasAndVoltageParams [pt_negVF] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negVF]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negVF << "DatawithADR: " << biasAndVoltageParams [pt_negVF] << "Data: " << (biasAndVoltageParams [pt_negVF] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateNegVFGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negVF] & 0xfff) * biasAndVoltageRange[pt_negVF]) / 4096.0;
    ui->doubleSpinBoxNegVF->setValue(eng_value);
}
void biasclocksgui::updateNegVFGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negVF] = (biasAndVoltageParams [pt_negVF] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negVF]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negVF] & 0xfff) * biasAndVoltageRange[pt_negVF]) / 4096.0;
    ui->doubleSpinBoxNegVF->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosVF_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posVF] = (biasAndVoltageParams [pt_posVF] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posVF]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posVF << "DatawithADR: " << biasAndVoltageParams [pt_posVF] << "Data: " << (biasAndVoltageParams [pt_posVF] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updatePosVFGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posVF] & 0xfff) * biasAndVoltageRange[pt_posVF]) / 4096.0;
    ui->doubleSpinBoxPosVF->setValue(eng_value);
}
void biasclocksgui::updatePosVFGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posVF] = (biasAndVoltageParams [pt_posVF] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posVF]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posVF] & 0xfff) * biasAndVoltageRange[pt_posVF]) / 4096.0;
    ui->doubleSpinBoxPosVF->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxNegTG_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negTG] = (biasAndVoltageParams [pt_negTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negTG]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negTG << "DatawithADR: " << biasAndVoltageParams [pt_negTG] << "Data: " << (biasAndVoltageParams [pt_negTG] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateNegTGGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negTG] & 0xfff) * biasAndVoltageRange[pt_negTG]) / 4096.0;
    ui->doubleSpinBoxNegTG->setValue(eng_value);

}
void biasclocksgui::updateNegTGGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negTG] = (biasAndVoltageParams [pt_negTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negTG]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negTG] & 0xfff) * biasAndVoltageRange[pt_negTG]) / 4096.0;
    ui->doubleSpinBoxNegTG->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosTG_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posTG] = (biasAndVoltageParams [pt_posTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posTG]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posTG << "DatawithADR: " << biasAndVoltageParams [pt_posTG] << "Data: " << (biasAndVoltageParams [pt_posTG] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}
void biasclocksgui::updatePosTGGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posTG] & 0xfff) * biasAndVoltageRange[pt_posTG]) / 4096.0;
    ui->doubleSpinBoxPosTG->setValue(eng_value);
}
void biasclocksgui::updatePosTGGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posTG] = (biasAndVoltageParams [pt_posTG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posTG]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posTG] & 0xfff) * biasAndVoltageRange[pt_posTG]) / 4096.0;
    ui->doubleSpinBoxPosTG->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxNegH_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negH] = (biasAndVoltageParams [pt_negH] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negH]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negH << "DatawithADR: " << biasAndVoltageParams [pt_negH] << "Data: " << (biasAndVoltageParams [pt_negH] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}
void biasclocksgui::updateNegHGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negH] & 0xfff) * biasAndVoltageRange[pt_negH]) / 4096.0;
    ui->doubleSpinBoxNegH->setValue(eng_value);
}
void biasclocksgui::updateNegHGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negH] = (biasAndVoltageParams [pt_negH] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negH]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negH] & 0xfff) * biasAndVoltageRange[pt_negH]) / 4096.0;
    ui->doubleSpinBoxNegH->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosH_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posH] = (biasAndVoltageParams [pt_posH] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posH]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posH << "DatawithADR: " << biasAndVoltageParams [pt_posH] << "Data: " << (biasAndVoltageParams [pt_posH] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updatePosHGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posH] & 0xfff) * biasAndVoltageRange[pt_posH]) / 4096.0;
    ui->doubleSpinBoxPosH->setValue(eng_value);
}
void biasclocksgui::updatePosHGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posH] = (biasAndVoltageParams [pt_posH] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posH]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posH] & 0xfff) * biasAndVoltageRange[pt_posH]) / 4096.0;
    ui->doubleSpinBoxPosH->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxNegRG_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negRG] = (biasAndVoltageParams [pt_negRG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negRG]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negRG << "DatawithADR: " << biasAndVoltageParams [pt_OTG] << "Data: " << (biasAndVoltageParams [pt_negRG] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}
void biasclocksgui::updateNegRGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negRG] & 0xfff) * biasAndVoltageRange[pt_negRG]) / 4096.0;
    ui->doubleSpinBoxNegRG->setValue(eng_value);
}
void biasclocksgui::updateNegRGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negRG] = (biasAndVoltageParams [pt_negRG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negRG]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negRG] & 0xfff) * biasAndVoltageRange[pt_negRG]) / 4096.0;
    ui->doubleSpinBoxNegRG->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosRG_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posRG] = (biasAndVoltageParams [pt_posRG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posRG]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posRG << "DatawithADR: " << biasAndVoltageParams [pt_posRG] << "Data: " << (biasAndVoltageParams [pt_posRG] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updatePosRGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posRG] & 0xfff) * biasAndVoltageRange[pt_posRG]) / 4096.0;
    ui->doubleSpinBoxPosRG->setValue(eng_value);
}
void biasclocksgui::updatePosRGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posRG] = (biasAndVoltageParams [pt_posRG] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posRG]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posRG] & 0xfff) * biasAndVoltageRange[pt_posRG]) / 4096.0;
    ui->doubleSpinBoxPosRG->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxNegSW_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_negSW] = (biasAndVoltageParams [pt_negSW] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negSW]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_negSW << "DatawithADR: " << biasAndVoltageParams [pt_negSW] << "Data: " << (biasAndVoltageParams [pt_negSW] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updateNegSWGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_negSW] & 0xfff) * biasAndVoltageRange[pt_negSW]) / 4096.0;
    ui->doubleSpinBoxNegSW->setValue(eng_value);
}
void biasclocksgui::updateNegSWGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_negSW] = (biasAndVoltageParams [pt_negSW] & 0xc000) + ((arg1/biasAndVoltageRange[pt_negSW]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_negSW] & 0xfff) * biasAndVoltageRange[pt_negSW]) / 4096.0;
    ui->doubleSpinBoxNegSW->setValue(eng_value);
}

void biasclocksgui::on_doubleSpinBoxPosSW_valueChanged(double arg1)
{
    // convert to engineering unints and store
    biasAndVoltageParams [pt_posSW] = (biasAndVoltageParams [pt_posSW] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posSW]) * 0xfff);
    qDebug() << "Write Bias and Voltage Array    Address: " << pt_posSW << "DatawithADR: " << biasAndVoltageParams [pt_posSW] << "Data: " << (biasAndVoltageParams [pt_posSW] & 0xfff);
    ui->labelBiasesToCin->setVisible(1);
}

void biasclocksgui::updatePosSWGUIRemote(int addr, int data)
{
    qDebug() << "Remote OTG Cmd In ADDR:" << addr << "  Raw data" << data;
    setBiasAndVoltageParams(addr,data);
    float eng_value = float( (biasAndVoltageParams[pt_posSW] & 0xfff) * biasAndVoltageRange[pt_posSW]) / 4096.0;
    ui->doubleSpinBoxPosSW->setValue(eng_value);
}
void biasclocksgui::updatePosSWGUIRemote2(double arg1)
{
    biasAndVoltageParams [pt_posSW] = (biasAndVoltageParams [pt_posSW] & 0xc000) + ((arg1/biasAndVoltageRange[pt_posSW]) * 0xfff);
    float eng_value = float( (biasAndVoltageParams[pt_posSW] & 0xfff) * biasAndVoltageRange[pt_posSW]) / 4096.0;
    ui->doubleSpinBoxPosSW->setValue(eng_value);
}

