#include "triggerexposuregui.h"
#include "ui_triggerexposuregui.h"

#include "mainwindow.h"
#include "ui_mainwindow.h"

triggerexposuregui::triggerexposuregui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::triggerexposuregui)
{
    ui->setupUi(this);

    ui->pushButtonStartExp->show();
    ui->pushButtonStopExp->hide();

    ui->comboBoxTrigMode->show();

    triggerShell = new QProcess(this);
}

triggerexposuregui::~triggerexposuregui()
{
    delete ui;
}

//============================================================================
//              Remote Functions
//============================================================================

void triggerexposuregui::startTrigRemote()
{
    ui->pushButtonStartExp->click();
}

void triggerexposuregui::stopTrigRemote()
{
    ui->pushButtonStopExp->click();
}

void triggerexposuregui::setExpTime (double arg1)
{
    ui->doubleSpinBoxExpTime->setValue( arg1 );
}

/*
void triggerexposuregui::setExpTime2 (double arg1)
{
    ui->doubleSpinBoxExpTime2->setValue( arg1 );
}
*/

void triggerexposuregui::setExposureNumber ( int arg1 )
{
    ui->spinBoxNumExposures->setValue ( arg1 );
}

void triggerexposuregui::setTrgRepTime (int arg1)
{
    ui->spinBoxCycleTime->setValue( arg1 );
}

void triggerexposuregui::setShutterMode(int arg1)
{
    ui->comboBoxShutterMode->setCurrentIndex(arg1);
}

void triggerexposuregui::setExpMode(int arg1)
{
    ui->comboBoxExpMode->setCurrentIndex(arg1);
}

/*
void triggerexposuregui::setDoubleExpMode(int arg1)
{
    ui->comboBoxDoubleExpMode->setCurrentIndex(arg1);
}
*/

//============================================================================
//              Main Window Functions
//============================================================================

int triggerexposuregui::getExpMode()
{
    return ui->comboBoxExpMode->currentIndex();
}

int triggerexposuregui::getTrigMode()
{
    return ui->comboBoxTrigMode->currentIndex();
}

//============================================================================
// Utility Functions, I plan to move these to mainwindow.cpp to reduce code duplication. vmc
//============================================================================

//This library is copied from mainwindow.cpp. There is also one in bias clocks GUI
//We need to find a better way to do these wrappers without repeating the code. VMC

// QThread has static sleep functions; but they're protected (duh).
// So we provide wrapper functions:
//
//    void MyLib::sleep (unsigned long secs)
//    void MyLib::msleep (unsigned long msecs)
//    void MyLib::usleep (unsigned long usecs)
//#include <QThread>
namespace MyLib
{
class DerivedFromQThread : protected QThread
{
public:
    static void sleep (unsigned long secs);
    static void msleep (unsigned long msecs);
    static void usleep (unsigned long usecs);
} ;
void sleep (unsigned long secs);
void msleep (unsigned long msecs);
void usleep (unsigned long usecs);
} // namespace MyLib

// I want this function to exist in mainwindow and mainwindow only. vmc
// regAddress must be of the form 0x8iii
void triggerexposuregui::writeDatagram(int regAddress, int regData)
{
    QByteArray datagram;

    datagram.clear();
    append16(datagram, regAddress, regData);

    commandSocket->writeDatagram(datagram.data(), datagram.size(), *cameraControlIP, *cameraControlPortNum);
    MyLib::msleep(1);
}

struct FormattedOctet {
    int ms_data;
    int ls_data;
};

FormattedOctet triggerexposuregui::convertDecToHexForDatagram(int arg1)
{
    FormattedOctet octet;

    octet.ms_data = (arg1 & 0xffff0000) >> 16;
    octet.ls_data = arg1 & 0xffff;

    return octet;
}

FormattedOctet triggerexposuregui::convertExpTimeToHexForDatagram(double arg1)
{
    FormattedOctet octet;

    //bad type casting will live on for now. should fix later. vmc
    octet.ms_data = (int) ( (arg1*100.0) / 65536);
    octet.ls_data = (int) ( double (arg1*100.0)) % 65536;

    return octet;
}

//============================================================================
//              Drop Down Menus
//============================================================================


void triggerexposuregui::on_comboBoxTrigMode_activated(int index)
{
    //Update UI and call python script.
    if (index == 0) //Software Triggers
    {
        ui->comboBoxExpMode->show();
        ui->pushButtonStartExp->show();
        ui->label_63->show();
        cin->formatPythonCommand("setTriggerSW.py");
    }
    else if (index >= 1) //External (Hardware) Triggers
    {
        ui->comboBoxExpMode->hide();
        ui->pushButtonStartExp->hide();
        ui->pushButtonStopExp->hide();
        ui->label_63->hide();

        if (index == 1) cin->formatPythonCommand("setTrigger0.py");    //External trigger 1
        if (index == 2) cin->formatPythonCommand("setTrigger1.py");    //External trigger 2
        if (index == 3) cin->formatPythonCommand("setTriggerOR.py");   //External trigger 1 or 2
    }
    // qDebug()<< "Trigger signals being called with index: " << index;
}

void triggerexposuregui::formatAndSendTriggerToGUI(int focusMode, int numExp)
{
    // set focus to 0
    cin->clock_control_reg = cin->clock_control_reg & 0xfffd;
    writeDatagram(0x8205, cin->clock_control_reg);

    // set number of exposures
    writeDatagram(0x820c, numExp);

    if (focusMode == 1)
    {
        // set focus to 1
        cin->clock_control_reg = cin->clock_control_reg | 0x0002;
        writeDatagram(0x8205, cin->clock_control_reg);
    }
    else if (focusMode == 0)
    {
        // do nothing
    }
    else{
        console->append(QLatin1String("Tried to start triggering while in unknown focus mode. Focus mode was set to: " ) + QString("%1").arg(focusMode));
    }
}

//============================================================================
//              Push Buttons
//============================================================================

void triggerexposuregui::on_pushButtonStartExp_clicked()
{
    int exposureMode;
    int focusMode;
    int numExp;

    // First, sync the exposure and cycle time to the values shown in the GUI
    sendExposureTimeDatagram(ui->doubleSpinBoxExpTime->value());
    sendCycleTimeDatagram(ui->spinBoxCycleTime->value());
    //If we are in double exposure mode, send the second exposure time too.
//    sendExposureTime2Datagram(ui->doubleSpinBoxExpTime2->value());

    exposureMode = ui->comboBoxExpMode->currentIndex();
    //console->append(QLatin1String("Exposure mode is: " ) + QString("%1").arg(exposureMode));
    //Update the UI
    if (exposureMode == 0) //Single Trigger Mode
    {
        //Send single trigger
        focusMode = 0;
        numExp = 1;

        // format trigger register and send out command to trigger exposure or sync pulse
        triggerexposuregui::formatAndSendTriggerToGUI(focusMode, numExp);

	// send out command to trigger exposure or sync pulse
	writeDatagram(0x8001, 0x100);

        //Update UI
        ui->pushButtonStartExp->hide();
        MyLib::msleep(100);
        ui->pushButtonStartExp->show();
    }

    else if (exposureMode == 1) //Multiple Exposure Mode
    {
        focusMode = 1;
        numExp =  ui->spinBoxNumExposures->value();

        // format trigger register and send out command to trigger exposure or sync pulse
        triggerexposuregui::formatAndSendTriggerToGUI(focusMode, numExp);

	// send out command to trigger exposure or sync pulse
	writeDatagram(0x8001, 0x100);

        //Update UI
        ui->pushButtonStartExp->hide();
        ui->pushButtonStopExp->show();
    }

    else if (exposureMode == 2) //Continuous Exposure Mode
    {
        focusMode = 1;
        numExp = 0;

        // format trigger register and send out command to trigger exposure or sync pulse
        triggerexposuregui::formatAndSendTriggerToGUI(focusMode, numExp);

	// send out command to trigger exposure or sync pulse
	writeDatagram(0x8001, 0x100);

        //Update UI
        ui->pushButtonStartExp->hide();
        ui->pushButtonStopExp->show();
    }
    else
    {
        console->append(QLatin1String("Tried to start triggering while in unknown trigger mode. Exposure combo box index was set to: " ) + QString("%1").arg(exposureMode));
    }
}

void triggerexposuregui::on_pushButtonStopExp_clicked()
{
    // send out command to stop triggering exposure
    int exposureMode;
    int focusMode;
    int numExp;

    exposureMode = ui->comboBoxExpMode->currentIndex();
    if (exposureMode == 0 || exposureMode == 1 || exposureMode == 2) //Single or Multiple or Continuous Exposure Mode
    {
        focusMode = 0;
        numExp = 1;

        // format trigger register and send out command to trigger exposure or sync pulse
        triggerexposuregui::formatAndSendTriggerToGUI(focusMode, numExp);
    }
    else
    {
        console->append(QLatin1String("Tried to stop triggering while in unknown trigger mode. Exposure combo box index was set to: " ) + QString("%1").arg(exposureMode));
    }

    //Update the UI
    ui->pushButtonStopExp->hide();
    ui->pushButtonStartExp->show();
}

//============================================================================
//              Spin Box Value Changes
//============================================================================

void triggerexposuregui::on_comboBoxShutterMode_activated(int index)
{
    if (index == 0)     // index = 0 shutter is closed
    {
        console->append("Command: Shutter Closed Mode");
        // First clear shutter bits
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        // Next set to close shutter
        cin->clock_control_reg = cin->clock_control_reg | 0x8;

        writeDatagram(0x8205, cin->clock_control_reg);
    }

    if (index == 1)      // index = 1 shutter is open
    {
        console->append("Command: Shutter Open Mode");
        // First clear shutter bits
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        // Next set to open shutter
        cin->clock_control_reg = cin->clock_control_reg | 0xc;

        writeDatagram(0x8205, cin->clock_control_reg);
    }

    if (index == 2)    // index = 2 shutter is normal - open/closed depending on exposure control
    {
        console->append("Command: Shutter Normal(Active) Mode");

        // clear shutter bits
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;

        writeDatagram(0x8205, cin->clock_control_reg);
    }

    console->append(QLatin1String("ShutActive:" ) + QString("%1").arg(cin->clock_control_reg));
}

void triggerexposuregui::on_doubleSpinBoxExpTime_valueChanged(double arg1)
{
    // Sending the expsoure time datagram now happens when trigger expsure gui is pushed, so the code needs to be in a seperate function.
    sendExposureTimeDatagram(arg1);
}

void triggerexposuregui::sendExposureTimeDatagram(double arg1)
{
    // send out exposure time
    FormattedOctet data = convertExpTimeToHexForDatagram(arg1);

    writeDatagram(0x8206, data.ms_data);
    writeDatagram(0x8207, data.ls_data);

    console->append("Exposure Time MS = " + QString("%1").arg(data.ms_data) + "  LS = " + QString("%1").arg(data.ls_data) );
}

/*
void triggerexposuregui::on_doubleSpinBoxExpTime2_valueChanged(double arg1)
{
    // Sending the expsoure time datagram now happens when trigger expsure gui is pushed, so the code needs to be in a seperate function.
    sendExposureTime2Datagram(arg1);
}
*/

void triggerexposuregui::sendExposureTime2Datagram(double arg1)
{
    // send out exposure time
    FormattedOctet data = convertExpTimeToHexForDatagram(arg1);

    writeDatagram(0x8306, data.ms_data);
    writeDatagram(0x8307, data.ls_data);
}

void triggerexposuregui::on_spinBoxCycleTime_valueChanged(int arg1)
{
    // Similar to expsoure time, setting the cycle time datagram now happens when trigger expsure gui is pushed, so the code needs to be in a seperate function.
    sendCycleTimeDatagram(arg1);
}

void triggerexposuregui::sendCycleTimeDatagram(int arg1)
{
    // send out cycle time which must be >= Exposure time + Read Time
    FormattedOctet data = convertDecToHexForDatagram(arg1);

    writeDatagram(0x8208, data.ms_data);
    writeDatagram(0x8209, data.ls_data);

    console->append("Trigger repetition time MS = " + QString("%1").arg(data.ms_data) + "  LS = " + QString("%1").arg(data.ls_data) );
}

void triggerexposuregui::on_spinBoxDelayTtoE_valueChanged(int arg1)
{
    // send out delay time between Trigger and Exposure
    FormattedOctet data = convertDecToHexForDatagram(arg1);

    writeDatagram(0x820a, data.ms_data);
    writeDatagram(0x820b, data.ls_data);
}


void triggerexposuregui::on_spinBoxShutterTime_valueChanged(int arg1)
{
    // send out shutter time in msec
    FormattedOctet data = convertDecToHexForDatagram(arg1);

    writeDatagram(0x820d, data.ms_data);
    writeDatagram(0x820e, data.ls_data);
}

void triggerexposuregui::on_spinBoxDelayTtoS_valueChanged(int arg1)
{
    // send out Delay between exposure trigger and shutter starting
    FormattedOctet data = convertDecToHexForDatagram(arg1);

    writeDatagram(0x820f, data.ms_data);
    writeDatagram(0x8210, data.ls_data);
}

void triggerexposuregui::on_spinBoxNumExposures_valueChanged(int arg1)
{
    // send out number of exposures
    // jtw 4-10-2012 should be done like exposure time
    setExposureNumber(arg1);

    //    QString Line;
    //    QList<QString> SplitList;

    //    QByteArray datagram;

    //    int data;
    //    data = arg1 & 0xffff;
    //    datagram.clear();
    //    append16(datagram, 0x820c, data);
    //    commandSocket->writeDatagram(datagram.data(), datagram.size(), cameraControlIP, cameraControlPortNum);
    //    MyLib::msleep(1);
}

/*
void triggerexposuregui::on_comboBoxDoubleExpMode_activated(int index)
{

    if (index == 0)     // index = 0 disable double exposure
    {
        writeDatagram(0x8050, 0x0000);
    }

    if (index == 1)      // index = 1 even/odd
    {
        writeDatagram(0x8050, 0x8000);
    }

    if (index == 2)    // index = 2 exp/exp2 2/2
    {
        writeDatagram(0x8050, 0x8100);
    }

    if (index == 3)    // index = 2 exp/exp2 4/4
    {
        writeDatagram(0x8050, 0x8200);
    }

    if (index == 4)    // index = 2 exp/exp2 8/8
    {
        writeDatagram(0x8050, 0x8300);
    }

    if (index == 5)    // index = 2 exp/exp2 16/16
    {
        writeDatagram(0x8050, 0x8400);
    }

    if (index == 6)    // index = 2 exp/exp2 32/32
    {
        writeDatagram(0x8050, 0x8500);
    }

    if (index == 7)    // index = 2 exp/exp2 64/64
    {
        writeDatagram(0x8050, 0x8600);
    }

    if (index == 8)    // index = 2 exp/exp2 128/128
    {
        writeDatagram(0x8050, 0x8700);
    }
}
*/

void triggerexposuregui::UpdateGUITriggerExposureSettings( int arg1 )
{
    //ui->comboBoxExpMode->setValidator();
    if ( (arg1 & 0x2) == 0x0)
    {
        cin->clock_control_reg = cin->clock_control_reg & 0xfffd;
        console->append("Single Trigger Mode");
        ui->comboBoxExpMode->setCurrentIndex(0);
    }
    else if ( (arg1 & 0x2) == 0x2)
    {
        cin->clock_control_reg = cin->clock_control_reg | 0x0002;
        console->append("Multiple/Continuous Trigger Mode");
        ui->comboBoxExpMode->setCurrentIndex(2);
    }

    if ( (arg1 & 0xc) == 0x0)
    {
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        console->append("Shutter Normal(Active) Mode");
        ui->comboBoxShutterMode->setCurrentIndex(2);
    }
    else if ( (arg1 & 0xc) == 0x8)
    {
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        cin->clock_control_reg = cin->clock_control_reg | 0x0008;
        console->append("Shutter Closed Mode");
        ui->comboBoxShutterMode->setCurrentIndex(0);
    }
    else if ( (arg1 & 0xc) == 0xc)
    {
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        cin->clock_control_reg = cin->clock_control_reg | 0x000c;
        console->append("Shutter Open Mode");
        ui->comboBoxShutterMode->setCurrentIndex(1);

    }
    else if ( (arg1 & 0xc) == 0x4)
    {
        cin->clock_control_reg = cin->clock_control_reg & 0xfff3;
        cin->clock_control_reg = cin->clock_control_reg | 0x0004;
        console->append("ERROR: Invalid Shutter Mode!");
    }

    console->append(QString("Clock and control register is set to: ").arg(cin->clock_control_reg));
}
